<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Magento\Swatches\Test\Unit\Model\Plugin;

use Magento\Catalog\Model\ResourceModel\Eav\Attribute;
use Magento\Eav\Model\Entity\Attribute\Source\AbstractSource;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use Magento\Swatches\Helper\Data;
use Magento\Swatches\Model\Plugin\EavAttribute;
use Magento\Swatches\Model\ResourceModel\Swatch\Collection;
use Magento\Swatches\Model\ResourceModel\Swatch\CollectionFactory;
use Magento\Swatches\Model\Swatch;
use Magento\Swatches\Model\SwatchAttributeType;
use Magento\Swatches\Model\SwatchFactory;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

/**
 * Test plugin model for Catalog Resource Attribute
 *
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 * @SuppressWarnings(PHPMD.TooManyPublicMethods)
 * @SuppressWarnings(PHPMD.ExcessiveClassLength)
 */
class EavAttributeTest extends TestCase
{
    private const ATTRIBUTE_ID = 123;
    private const OPTION_1_ID = 1;
    private const OPTION_2_ID = 2;
    private const ADMIN_STORE_ID = 0;
    private const DEFAULT_STORE_ID = 1;
    private const NEW_OPTION_KEY = 'option_2';
    private const ATTRIBUTE_DEFAULT_VALUE = [
        0 => self::NEW_OPTION_KEY
    ];
    private const VISUAL_ATTRIBUTE_OPTIONS = [
        'value' => [
            self::OPTION_1_ID => [
                self::ADMIN_STORE_ID => 'Black',
                self::DEFAULT_STORE_ID => 'Black',
            ],
            self::NEW_OPTION_KEY => [
                self::ADMIN_STORE_ID => 'White',
                self::DEFAULT_STORE_ID => 'White',
            ],
        ]
    ];
    private const VISUAL_SWATCH_OPTIONS = [
        'value' => [
            self::OPTION_1_ID => '#000000',
            self::NEW_OPTION_KEY => '#ffffff',
        ]
    ];
    private const VISUAL_SAVED_OPTIONS = [
        [
            'value' => self::OPTION_1_ID,
            'label' => 'Black',
        ],
        [
            'value' => self::OPTION_2_ID,
            'label' => 'White',
        ]
    ];
    private const TEXT_ATTRIBUTE_OPTIONS = [
        'value' => [
            self::OPTION_1_ID => [
                self::ADMIN_STORE_ID => 'Small',
                self::DEFAULT_STORE_ID => 'Small',
            ],
            self::NEW_OPTION_KEY => [
                self::ADMIN_STORE_ID => 'Medium',
                self::DEFAULT_STORE_ID => 'Medium',
            ],
        ]
    ];
    private const TEXT_SWATCH_OPTIONS = [
        'value' => [
            self::OPTION_1_ID => [
                self::ADMIN_STORE_ID => 'S',
                self::DEFAULT_STORE_ID => 'S',
            ],
            self::NEW_OPTION_KEY => [
                self::ADMIN_STORE_ID => 'M',
                self::DEFAULT_STORE_ID => 'M',
            ],
        ]
    ];
    private const TEXT_SAVED_OPTIONS = [
        [
            'value' => self::OPTION_1_ID,
            'label' => 'Small',
        ],
        [
            'value' => self::OPTION_2_ID,
            'label' => 'Medium',
        ]
    ];

    /** @var EavAttribute */
    private $eavAttribute;

    /** @var Attribute|MockObject */
    private $attribute;

    /** @var SwatchFactory|MockObject */
    private $swatchFactory;

    /** @var CollectionFactory|MockObject */
    private $collectionFactory;

    /** @var Data|MockObject */
    private $swatchHelper;

    /** @var AbstractSource|MockObject */
    private $abstractSource;

    /** @var \Magento\Swatches\Model\ResourceModel\Swatch|MockObject */
    private $resource;

    /** @var Collection|MockObject */
    private $collection;

    /**
     * {@inheritDoc}
     */
    protected function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->abstractSource = $this->createMock(AbstractSource::class);
        $this->attribute = $this->createPartialMock(
            Attribute::class,
            ['getSource']
        );
        $this->attribute->setId(self::ATTRIBUTE_ID);
        $this->swatchFactory = $this->createPartialMock(
            SwatchFactory::class,
            ['create']
        );
        $this->swatchHelper = $objectManager->getObject(
            Data::class,
            [
                'swatchTypeChecker' => $objectManager->getObject(SwatchAttributeType::class)
            ]
        );
        $this->resource = $this->createMock(\Magento\Swatches\Model\ResourceModel\Swatch::class);
        $this->collection = $this->createMock(Collection::class);
        $this->collectionFactory = $this->createPartialMock(CollectionFactory::class, ['create']);
        $serializer = $objectManager->getObject(Json::class);
        $this->eavAttribute = $objectManager->getObject(
            EavAttribute::class,
            [
                'collectionFactory' => $this->collectionFactory,
                'swatchFactory' => $this->swatchFactory,
                'swatchHelper' => $this->swatchHelper,
                'serializer' => $serializer,
            ]
        );
        $this->attribute->expects($this->any())
            ->method('getSource')
            ->willReturn($this->abstractSource);
        $swatch = $this->createMock(Swatch::class);
        $swatch->expects($this->any())
            ->method('getResource')
            ->willReturn($this->resource);
        $this->swatchFactory->expects($this->any())
            ->method('create')
            ->willReturn($swatch);
    }

    /**
     * Test beforeSave plugin for visual swatch
     */
    public function testBeforeSaveVisualSwatch()
    {
        $this->attribute->setData(
            [
                'defaultvisual' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optionvisual' => self::VISUAL_ATTRIBUTE_OPTIONS,
                'swatchvisual' => self::VISUAL_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_VISUAL);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
        $this->assertEquals(self::ATTRIBUTE_DEFAULT_VALUE, $this->attribute->getData('default'));
        $this->assertEquals(self::VISUAL_ATTRIBUTE_OPTIONS, $this->attribute->getData('option'));
        $this->assertEquals(self::VISUAL_SWATCH_OPTIONS, $this->attribute->getData('swatch'));
    }

    /**
     * Test beforeSave plugin for text swatch
     */
    public function testBeforeSaveTextSwatch()
    {
        $this->attribute->setData(
            [
                'defaulttext' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optiontext' => self::TEXT_ATTRIBUTE_OPTIONS,
                'swatchtext' => self::TEXT_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_TEXT);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
        $this->assertEquals(self::ATTRIBUTE_DEFAULT_VALUE, $this->attribute->getData('default'));
        $this->assertEquals(self::TEXT_ATTRIBUTE_OPTIONS, $this->attribute->getData('option'));
        $this->assertEquals(self::TEXT_SWATCH_OPTIONS, $this->attribute->getData('swatch'));
    }

    /**
     * Test beforeSave plugin on empty label
     *
     */
    public function testBeforeSaveWithFailedValidation()
    {
        $this->expectException(\Magento\Framework\Exception\InputException::class);
        $this->expectExceptionMessage('Admin is a required field in each row');

        $options = self::VISUAL_ATTRIBUTE_OPTIONS;
        $options['value'][self::NEW_OPTION_KEY][self::ADMIN_STORE_ID] = '';
        $this->attribute->setData(
            [
                'defaultvisual' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optionvisual' => $options,
                'swatchvisual' => self::VISUAL_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_VISUAL);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
    }

    /**
     * Test beforeSave plugin on empty label of option being deleted
     */
    public function testValidationIsSkippedForDeletedOption()
    {
        $options = self::VISUAL_ATTRIBUTE_OPTIONS;
        $options['value'][self::NEW_OPTION_KEY][self::ADMIN_STORE_ID] = '';
        $options['delete'][self::NEW_OPTION_KEY] = '1';
        $this->attribute->setData(
            [
                'defaultvisual' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optionvisual' => $options,
                'swatchvisual' => self::VISUAL_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_VISUAL);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
        $this->assertEquals(self::ATTRIBUTE_DEFAULT_VALUE, $this->attribute->getData('default'));
        $this->assertEquals($options, $this->attribute->getData('option'));
        $this->assertEquals(self::VISUAL_SWATCH_OPTIONS, $this->attribute->getData('swatch'));
    }

    /**
     * Test beforeSave plugin for non a swatch attribute
     */
    public function testBeforeSaveNotSwatch()
    {
        $additionalData = [
            Swatch::SWATCH_INPUT_TYPE_KEY => Swatch::SWATCH_INPUT_TYPE_VISUAL,
            'update_product_preview_image' => 1,
            'use_product_image_for_swatch' => 0
        ];

        $this->attribute->setData(
            [
                Swatch::SWATCH_INPUT_TYPE_KEY => Swatch::SWATCH_INPUT_TYPE_DROPDOWN,
                'additional_data' => json_encode($additionalData),
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_DROPDOWN);

        $this->eavAttribute->beforeBeforeSave($this->attribute);

        unset($additionalData[Swatch::SWATCH_INPUT_TYPE_KEY]);

        $this->assertEquals(json_encode($additionalData), $this->attribute->getData('additional_data'));
    }

    /**
     * @return array
     */
    public function visualSwatchProvider()
    {
        return [
            [Swatch::SWATCH_TYPE_EMPTY, 'black', 'white'],
            [Swatch::SWATCH_TYPE_VISUAL_COLOR, '#000000', '#ffffff'],
            [Swatch::SWATCH_TYPE_VISUAL_IMAGE, '/path/black.png', '/path/white.png'],
        ];
    }

    /**
     * Test afterSave plugin for visual swatch
     *
     * @param int $swatchType
     * @param string $swatch1
     * @param string $swatch2
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     * @dataProvider visualSwatchProvider
     */
    public function testAfterAfterSaveVisualSwatch(int $swatchType, string $swatch1, string $swatch2)
    {
        $options = self::VISUAL_SWATCH_OPTIONS;
        $options['value'][self::OPTION_1_ID] = $swatch1;
        $options['value'][self::NEW_OPTION_KEY] = $swatch2;
        $this->attribute->addData(
            [
                'defaultvisual' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optionvisual' => self::VISUAL_ATTRIBUTE_OPTIONS,
                'swatchvisual' => $options,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_VISUAL);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
        $this->abstractSource->expects($this->once())
            ->method('getAllOptions')
            ->willReturn(self::VISUAL_SAVED_OPTIONS);

        $this->resource->expects($this->once())
            ->method('saveDefaultSwatchOption')
            ->with(self::ATTRIBUTE_ID, self::OPTION_2_ID);

        $this->collection->expects($this->exactly(4))
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['option_id', self::OPTION_1_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::ADMIN_STORE_ID]
            )
            ->willReturnSelf();

        $this->collection->expects($this->exactly(2))
            ->method('getFirstItem')
            ->willReturnOnConsecutiveCalls(
                $this->createSwatchMock(
                    $swatchType,
                    $swatch1,
                    1
                ),
                $this->createSwatchMock(
                    $swatchType,
                    $swatch2,
                    null,
                    self::OPTION_2_ID,
                    self::ADMIN_STORE_ID
                )
            );
        $this->collectionFactory->expects($this->exactly(2))
            ->method('create')
            ->willReturn($this->collection);

        $this->eavAttribute->afterAfterSave($this->attribute);
    }

    /**
     * Test afterSave plugin for text swatch
     */
    public function testAfterAfterSaveTextualSwatch()
    {
        $this->attribute->addData(
            [
                'defaulttext' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optiontext' => self::TEXT_ATTRIBUTE_OPTIONS,
                'swatchtext' => self::TEXT_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_TEXT);
        $this->eavAttribute->beforeBeforeSave($this->attribute);

        $this->abstractSource->expects($this->once())
            ->method('getAllOptions')
            ->willReturn(self::TEXT_SAVED_OPTIONS);

        $this->resource->expects($this->once())
            ->method('saveDefaultSwatchOption')
            ->with(self::ATTRIBUTE_ID, self::OPTION_2_ID);

        $this->collection->expects($this->exactly(8))
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['option_id', self::OPTION_1_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_1_ID],
                ['store_id', self::DEFAULT_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::DEFAULT_STORE_ID]
            )
            ->willReturnSelf();

        $this->collection->expects($this->exactly(4))
            ->method('getFirstItem')
            ->willReturnOnConsecutiveCalls(
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::OPTION_1_ID][self::ADMIN_STORE_ID],
                    1
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::OPTION_1_ID][self::DEFAULT_STORE_ID],
                    1
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::NEW_OPTION_KEY][self::ADMIN_STORE_ID],
                    null,
                    self::OPTION_2_ID,
                    self::ADMIN_STORE_ID
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::NEW_OPTION_KEY][self::DEFAULT_STORE_ID],
                    null,
                    self::OPTION_2_ID,
                    self::DEFAULT_STORE_ID
                )
            );
        $this->collectionFactory->expects($this->exactly(4))
            ->method('create')
            ->willReturn($this->collection);

        $this->eavAttribute->afterAfterSave($this->attribute);
    }

    /**
     * Test afterSave plugin for deleted visual swatch option
     */
    public function testAfterAfterSaveVisualSwatchIsDelete()
    {
        $options = self::VISUAL_ATTRIBUTE_OPTIONS;
        $options['delete'][self::OPTION_1_ID] = '1';
        $this->attribute->addData(
            [
                'defaultvisual' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optionvisual' => $options,
                'swatchvisual' => self::VISUAL_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_VISUAL);
        $this->eavAttribute->beforeBeforeSave($this->attribute);
        $this->abstractSource->expects($this->once())
            ->method('getAllOptions')
            ->willReturn(self::VISUAL_SAVED_OPTIONS);

        $this->resource->expects($this->once())
            ->method('saveDefaultSwatchOption')
            ->with(self::ATTRIBUTE_ID, self::OPTION_2_ID);

        $this->collection->expects($this->exactly(2))
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::ADMIN_STORE_ID]
            )
            ->willReturnSelf();

        $this->collection->expects($this->exactly(1))
            ->method('getFirstItem')
            ->willReturnOnConsecutiveCalls(
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_VISUAL_COLOR,
                    self::VISUAL_SWATCH_OPTIONS['value'][self::NEW_OPTION_KEY],
                    null,
                    self::OPTION_2_ID,
                    self::ADMIN_STORE_ID
                )
            );
        $this->collectionFactory->expects($this->exactly(1))
            ->method('create')
            ->willReturn($this->collection);

        $this->eavAttribute->afterAfterSave($this->attribute);
    }

    /**
     * Test afterSave plugin for deleted text swatch option
     */
    public function testAfterAfterSaveTextualSwatchIsDelete()
    {
        $options = self::TEXT_ATTRIBUTE_OPTIONS;
        $options['delete'][self::OPTION_1_ID] = '1';
        $this->attribute->addData(
            [
                'defaulttext' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optiontext' => $options,
                'swatchtext' => self::TEXT_SWATCH_OPTIONS,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_TEXT);
        $this->eavAttribute->beforeBeforeSave($this->attribute);

        $this->abstractSource->expects($this->once())
            ->method('getAllOptions')
            ->willReturn(self::TEXT_SAVED_OPTIONS);

        $this->resource->expects($this->once())
            ->method('saveDefaultSwatchOption')
            ->with(self::ATTRIBUTE_ID, self::OPTION_2_ID);

        $this->collection->expects($this->exactly(4))
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::DEFAULT_STORE_ID]
            )
            ->willReturnSelf();

        $this->collection->expects($this->exactly(2))
            ->method('getFirstItem')
            ->willReturnOnConsecutiveCalls(
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::NEW_OPTION_KEY][self::ADMIN_STORE_ID],
                    null,
                    self::OPTION_2_ID,
                    self::ADMIN_STORE_ID
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    self::TEXT_SWATCH_OPTIONS['value'][self::NEW_OPTION_KEY][self::DEFAULT_STORE_ID],
                    null,
                    self::OPTION_2_ID,
                    self::DEFAULT_STORE_ID
                )
            );
        $this->collectionFactory->expects($this->exactly(2))
            ->method('create')
            ->willReturn($this->collection);

        $this->eavAttribute->afterAfterSave($this->attribute);
    }

    /**
     * Test afterSave plugin on empty swatch value
     */
    public function testAfterAfterSaveNotSwatchAttribute()
    {
        $options = self::TEXT_SWATCH_OPTIONS;
        $options['value'][self::OPTION_1_ID][self::ADMIN_STORE_ID] = null;
        $options['value'][self::OPTION_1_ID][self::DEFAULT_STORE_ID] = null;
        $options['value'][self::NEW_OPTION_KEY][self::ADMIN_STORE_ID] = null;
        $options['value'][self::NEW_OPTION_KEY][self::DEFAULT_STORE_ID] = null;
        $this->attribute->addData(
            [
                'defaulttext' => self::ATTRIBUTE_DEFAULT_VALUE,
                'optiontext' => self::TEXT_ATTRIBUTE_OPTIONS,
                'swatchtext' => $options,
            ]
        );

        $this->attribute->setData(Swatch::SWATCH_INPUT_TYPE_KEY, Swatch::SWATCH_INPUT_TYPE_TEXT);
        $this->eavAttribute->beforeBeforeSave($this->attribute);

        $this->abstractSource->expects($this->once())
            ->method('getAllOptions')
            ->willReturn(self::TEXT_SAVED_OPTIONS);

        $this->resource->expects($this->once())
            ->method('saveDefaultSwatchOption')
            ->with(self::ATTRIBUTE_ID, self::OPTION_2_ID);

        $this->collection->expects($this->exactly(8))
            ->method('addFieldToFilter')
            ->withConsecutive(
                ['option_id', self::OPTION_1_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_1_ID],
                ['store_id', self::DEFAULT_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::ADMIN_STORE_ID],
                ['option_id', self::OPTION_2_ID],
                ['store_id', self::DEFAULT_STORE_ID]
            )
            ->willReturnSelf();

        $this->collection->expects($this->exactly(4))
            ->method('getFirstItem')
            ->willReturnOnConsecutiveCalls(
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    null,
                    1
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    null,
                    1
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    null,
                    null,
                    self::OPTION_2_ID,
                    self::ADMIN_STORE_ID
                ),
                $this->createSwatchMock(
                    Swatch::SWATCH_TYPE_TEXTUAL,
                    null,
                    null,
                    self::OPTION_2_ID,
                    self::DEFAULT_STORE_ID
                )
            );
        $this->collectionFactory->expects($this->exactly(4))
            ->method('create')
            ->willReturn($this->collection);

        $this->eavAttribute->afterAfterSave($this->attribute);
    }

    /**
     * Create configured mock for swatch model
     *
     * @param string $type
     * @param string|null $value
     * @param int|null $id
     * @param int|null $optionId
     * @param int|null $storeId
     * @return MockObject
     */
    private function createSwatchMock(
        string $type,
        ?string $value,
        ?int $id = null,
        ?int $optionId = null,
        ?int $storeId = null
    ) {
        $swatch = $this->createMock(Swatch::class);
        $swatch->expects($this->any())
            ->method('getId')
            ->willReturn($id);
        $swatch->expects($this->any())
            ->method('getResource')
            ->willReturn($this->resource);
        $swatch->expects($this->once())
            ->method('save');
        if ($id) {
            $swatch->expects($this->exactly(2))
                ->method('setData')
                ->withConsecutive(
                    ['type', $type],
                    ['value', $value]
                );
        } else {
            $swatch->expects($this->exactly(4))
                ->method('setData')
                ->withConsecutive(
                    ['option_id', $optionId],
                    ['store_id', $storeId],
                    ['type', $type],
                    ['value', $value]
                );
        }
        return $swatch;
    }
}
